<?php

/**
 * @file
 * Contains a BlockPluginFactory.
 *
 * @license GPL v2 http://www.fsf.org/licensing/licenses/gpl.html
 * @author Chris Skene chris at previousnext dot com dot au
 * @copyright Copyright(c) 2014 Previous Next Pty Ltd
 */

namespace Drupal\ghost\Block;

use Drupal\ghost\Exception\InvalidPluginException;
use Drupal\ghost\Block\Plugin\BlockPluginInterface;

/**
 * Class BlockPluginFactory
 *
 * @package Drupal\ghost\Block
 */
class BlockPluginFactory {

  /**
   * Constructor.
   *
   * @return BlockPluginFactory
   *   This factory.
   */
  public function __construct() {

    return $this;
  }

  /**
   * Constructor.
   *
   * @return BlockPluginFactory
   *   This factory.
   */
  static public function factory() {

    return new static();
  }

  /**
   * Load an individual plugin.
   *
   * @param string $name
   *   Name of the plugin.
   *
   * @return bool|BlockPluginInterface
   *   a Block Plugin, or FALSE.
   */
  public function loadPlugin($name) {

    $definition = $this->loadPluginDefinition($name);
    if (empty($definition)) {

      return FALSE;
    }

    return $this->createPlugin($definition);
  }

  /**
   * Create a plugin.
   *
   * @param array $definition
   *   The ctools plugin definition.
   *
   * @return BlockPluginInterface|null
   *   A Block Plugin class.
   */
  public function createPlugin($definition) {
    $plugin = NULL;

    if (isset($definition['class']) && class_exists($definition['class'])) {
      try {
        $plugin = new $definition['class']($definition);

        if (!$plugin instanceof BlockPluginInterface) {
          throw new InvalidPluginException();
        }
      }
      catch (\Exception $e) {
        watchdog_exception('block_plugin', $e, $e->getMessage());
      }
    }

    return $plugin;
  }

  /**
   * Load all block_plugin plugins.
   *
   * @return BlockPluginInterface[]
   *   An array of plugins
   */
  public function loadAllPlugins() {
    $plugins = array();

    $definitions = $this->loadAllPluginDefinitions();

    if (empty($definitions)) {
      return $plugins;
    }

    foreach ($definitions as $definition) {
      $plugin = $this->createPlugin($definition);

      if (!empty($plugin)) {
        $plugins[$plugin->getMachineName()] = $plugin;
      }
    }

    return $plugins;
  }

  /**
   * Load a block_plugin plugin.
   *
   * @param string $name
   *   Name of the plugin
   *
   * @return array
   *   An array of plugin information
   */
  public function loadPluginDefinition($name) {

    ctools_include('plugins');

    return ctools_get_plugins('ghost', GHOST_CTOOLS_TYPE_BLOCK, $name);
  }

  /**
   * Load all block_plugin plugins.
   *
   * @return array
   *   An array of plugin information
   */
  public function loadAllPluginDefinitions() {

    ctools_include('plugins');

    return ctools_get_plugins('ghost', GHOST_CTOOLS_TYPE_BLOCK);
  }
}
